<?php
/*
Plugin Name: Esponce QR Code Generator
Plugin URI: http://www.esponce.com/plugins/wordpress/
Description: Generates a QR Code and inserts it into a blog post (shortcode) or sidebar widget. User can edit content, set module and background color, size, padding, version, etc. Plugin uses Esponce QR Code API 3.0 to generate a QR Code image. Image is then cached on the local WordPress server.
Version: 1.4
Author: Esponce.com
Author URI: http://www.esponce.com/
License: MIT License
*/

/*
History:
1.4  Widget and shortcode: extra options and bug fixes
1.3  Widget: added caption and permalink
1.2  Added caching
1.1  Improved widget and shortcode
1.0  Initial release
*/

/***** BEGIN MIT LICENSE BLOCK *****
 *
 * Copyright (C) 2012 Esponce d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ***** END MIT LICENCE BLOCK *****/
 
//User-defined constants

//Include URL query in permalink: "true" to add query strings (larger QR code), "false" for no query strings (smaller QR code)
define("ESPONCE_PERMALINK_WITH_QUERY", false);

//Enable caching: "true" to enable caching (more reliable and best performance), "false" to always get from web service (frequent content changes); note that this parameter is applied on save when editing a blog post
define("ESPONCE_ENABLE_CACHING", true);

//Imports
require_once('esp-api.php');
require_once('esp-helper.php');
require_once('esp-cache.php');
require_once('esp-widget.php');

/**************************************************************************************//**
 * Initialization
 ******************************************************************************************/
function esponce_init()
{
	//esponce_log("init");
}

function esponce_register_widget($parameter = null)
{
	return register_widget("EsponceQRCodeWidget");
}

/**************************************************************************************//**
 * Shortcode sytnax: [qrcode content="" size="" padding="" foreground="" background=""]
 ******************************************************************************************/
function esponce_shortcode($atts, $message = null)
{
	//Define default values
	$definition = array
	(
		'content' => '',
		'caption' => null,
		'tooltip' => null,
		'alt' => null,
		'class' => null,
		'style' => null,
		'width' => null,
		'height' => null,
		'format' => 'png',
		'version' => null,
		'padding' => null,
		'size' => null,
		'em' => null,
		'ec' => null,
		'foreground' => null,
		'background' => null
	);
	
	//Get attributes
	$a = shortcode_atts($definition, $atts);
	$content = esponce_decode_shortcode_value($a['content']);
	$caption = esponce_decode_shortcode_value($a['caption']);
	$tooltip = esponce_decode_shortcode_value($a['tooltip']);
	$alt = esponce_decode_shortcode_value($a['alt']);
	$class = $a['class'];
	$style = $a['style'];
	$width = $a['width'];
	$height = $a['height'];
	$format = $a['format'];
	$version = $a['version'];
	$padding = $a['padding'];
	$size = $a['size'];
	$em = $a['em'];
	$ec = $a['ec'];
	$foreground = $a['foreground'];
	$background = $a['background'];

	//Build URL to QR Code image
	$api = new EsponceQRCodeAPI();
	$url = $api->getURL($content, $format, $version, $size, $padding, $em, $ec, $foreground, $background);
	
	//Build HTML image
	$html = esponce_get_shortcode_html($url, $alt, $tooltip, $class, $style, $width, $height);

	//Return (and do not echo)
	return $html;
}

/**************************************************************************************//**
 * Creates cached images when a post is saved.
 ******************************************************************************************/
function esponce_save_post($post_id)
{
	try
	{
		if (ESPONCE_ENABLE_CACHING && !wp_is_post_revision($post_id))
		{
			$post = get_post($post_id);
			$content = $post->post_content;
			//esponce_log($content);
			
			$matches = null;
			preg_match_all('/\[qrcode(.*?)\]/i', $content, $matches);
			//esponce_log(print_r($matches, true));
			
			foreach ($matches[0] as $count => $shortcode)
			{
				//Download and cache image on local server
				$filename = esponce_cache_get_filename($shortcode);
				$url = esponce_get_url_from_shortcode($shortcode);
				esponce_cache_save_image($filename, $url);
			}
		}
	}
	catch (Exception $ex)
	{
		//esponce_log($ex->getMessage());
	}
}

/**************************************************************************************//**
 * Adds features to TinyMCE editor
 ******************************************************************************************/
function esponce_tinymce_button($buttons)
{
    array_push($buttons, "separator", "inject_esponce_qrcode");
    return $buttons;
}

function esponce_tinymce_register($plugins)
{
	$url = plugins_url("esponce-qr-code-generator/tinymce/plugin.js?su=" . urlencode(site_url("/")) . "&au=" . urlencode(admin_url())); //TODO: Fix includes_url() http://codex.wordpress.org/Function_Reference/wp_upload_dir
    $plugins["inject_esponce_qrcode"] = $url;
    return $plugins;
}

/**************************************************************************************//**
 * Register WordPress actions, filters and shortcodes
 ******************************************************************************************/
add_action('init', 'esponce_init');
add_action('save_post', 'esponce_save_post');
add_action('widgets_init', 'esponce_register_widget');
add_filter('the_content', 'esponce_cache_content_filter');
add_filter('mce_external_plugins', 'esponce_tinymce_register');
add_filter('mce_buttons', 'esponce_tinymce_button', 0);
add_shortcode('qrcode', 'esponce_shortcode');

?>